@@ -26,7 +26,7 @@ use strict;
 use warnings;
 
 use base qw/LWP::UserAgent/;
-use JSON::XS;
+use JSON;
 
 our $VERSION = '1.0';
 
@@ -65,9 +65,8 @@ sub new
 	# Not backed by a file yet
 	$self->cookie_jar ({});
 	# Prefer JSON, but deal with whatever else comes in, instead of letting backend return 406s
-	# XML is to make Backend think we're a browser, and redirect us upon token expirations
 	$self->default_header (Accept =>
-		'application/json;q=0.9, text/plain;q=0.2, application/xhtml+xml;q=0.1, */*;q=0.1');
+		'application/json;q=0.9, text/plain;q=0.2, */*;q=0.1');
 	return $self;
 }
 
@@ -91,6 +90,26 @@ sub post
 	return $self->SUPER::post ($uri, @args);
 }
 
+=item put URI, BODY, PARAMS
+
+Constructs and issues a PUT request.
+
+Compared to stock L<LWP::UserAgent>, the extra body parameter
+is encoded into JSON and set as request content, which is the
+only way to set the request content.
+
+The rest of parameters are passed to L<LWP::UserAgent> untouched.
+
+=cut
+
+sub put
+{
+	my ($self, $uri, $body, @args) = @_;
+	push @args,'Content-Type' => 'application/json',
+		Content => encode_json ($body);
+	return $self->SUPER::put ($uri, @args);
+}
+
 =item delete URI
 
 Convenience method for constructing and issuing a DELETE request.
@@ -129,8 +148,8 @@ sub request
 	return $response if ref $response eq 'HASH';
 
 	# Decode
-	my $decoded = eval { new JSON::XS->allow_nonref->decode ($response->content) }
-		if $response->header ('Content-Type') eq 'application/json';
+	my $decoded = eval { decode_json ($response->content) }
+		if $response->header ('Content-Type') =~ /^application\/json(;.*)?/;
 	$decoded = {
 		type => $response->header ('Content-Type'),
 		raw => $response->content,
@@ -140,8 +159,10 @@ sub request
 	unless ($response->is_success) {
 		# Apache::Error exceptions lack error wrapper
 		$decoded = $decoded->{error} if exists $decoded->{error};
-		die $response->status_line unless exists $decoded->{message};
-		die sprintf $decoded->{message}, @{$decoded->{parameters}};
+		my $request_id = $response->header ('X-GDC-Request') || "";
+		$request_id = " (Request ID: $request_id)" if $request_id;
+		die $response->status_line.$request_id unless exists $decoded->{message};
+		die sprintf ($decoded->{message}, @{$decoded->{parameters}}).$request_id;
 	}
 
 	return $decoded;
